==> XMLObject-0.1.1.tar.gz: XMLObject-0.1.1/XMLObject/TypedList.py <==
# Copyright (C) 2004 Philippe Normand <phil@respyre.org>
#
# This file is part of XMLObject (http://xmlobject.base-art.net)
#
# Under PSF License (see COPYING)

import UserList, copy
from main import XMLObject
import classregistry

class TypedList(UserList.UserList):
    """ A *special* list implementation

        It stores XMLObjects all of the same type OR strings.
        Mainly used by `Nodes.ListNode`.

        Example:

          class Bar(XMLObject):
              pass
          
          class Foo(XMLObject):
              liste = ListNode('Bar')

        `liste` node will create a TypedList in background. This list
        will ensure all items it stores are `Bar` instances.
    """
    
    def __init__(self, xmlList, data=[]):
        UserList.UserList.__init__(self, data)
        self._xmlList = xmlList

    def walkOnXMLObject(self,xoName, registry, callback, *args, **kw):
        theClass = classregistry.registry(registry).getClass(xoName)
        for nodeName, nodeInstance in theClass.__nodes__.iteritems():
            if nodeInstance.getType()  == 'Choice':
                for classRef in nodeInstance.getItemType():
                    if callback(classRef, *args, **kw):
                        return True
            elif nodeInstance.getType() in ['List','Item']:
                if callback(nodeInstance.getType(), *args, **kw):
                    return True
        return False
        
    def _compareTypes(self, itemType1, *args, **kw):
        """ Callback used to copare two type names.

            Used by `utils.walkOnXMLObject`
        """
        itemType2 = getattr(args[0],'__name__',args[0])
        if itemType1 == '#PCDATA' and type(itemType2) == type(''):
            itemType1 = itemType2
        return itemType1 == itemType2

    def checkItem(self, it):
        parentType = self._xmlList.getItemType()
        typeMismatch = False
        if type(it) == type(''):
            if not self.walkOnXMLObject(parentType,
                                        self._xmlList.getRegistry(),
                                        self._compareTypes, it):
                typeMismatch = True
     
        elif not isinstance(it, XMLObject):
            typeMismatch = True
        elif it.getName() != parentType and \
                 not self.walkOnXMLObject(parentType,
                                          self._xmlList.getRegistry(),
                                          self._compareTypes, it):
            typeMismatch = True
            
        if typeMismatch:
            raise TypeError("""\
%s type required for %s. Got %s instead""" % (`parentType`,
                                              `self._xmlList.getName()`,
                                              `type(it)`))
        return copy.deepcopy(it)
        
    def append(self, item):
        item = self.checkItem(item)
        UserList.UserList.append(self, item)

    def insert(self, index, item):
        item = self.checkItem(item)
        UserList.UserList.insert(self,index, item)

    def extend(self, other):
        other2 = []
        for it in other:
            other2.append(self.checkItem(it))
        UserList.UserList.extend(self, other2)

==> XMLObject-0.1.2.tar.gz: XMLObject-0.1.2/XMLObject/TypedList.py <==
# Copyright (C) 2004 Philippe Normand <phil@respyre.org>
#
# This file is part of XMLObject (http://xmlobject.base-art.net)
#
# Under PSF License (see COPYING)
# $HeadURL: https://svn.base-art.net/full/xmlobject/trunk/XMLObject/TypedList.py $
# $Id: TypedList.py 19 2004-08-15 15:47:30Z test $


import UserList, copy
from main import XMLObject
import classregistry

class TypedList(UserList.UserList):
    """ A *special* list implementation

        It stores XMLObjects all of the same type OR strings.
        Mainly used by `Nodes.ListNode`.

        Example:

          class Bar(XMLObject):
              pass
          
          class Foo(XMLObject):
              liste = ListNode('Bar')

        `liste` node will create a TypedList in background. This list
        will ensure all items it stores are `Bar` instances.
    """
    
    def __init__(self, xmlList, data=[]):
        UserList.UserList.__init__(self, data)
        self._xmlList = xmlList

    def walkOnXMLObject(self,xoName, registry, callback, *args, **kw):
        theClass = classregistry.registry(registry).getClass(xoName)
        for nodeName, nodeInstance in theClass.__nodes__.iteritems():
            if nodeInstance.getType()  == 'Choice':
                for classRef in nodeInstance.getItemType():
                    if callback(classRef, *args, **kw):
                        return True
            elif nodeInstance.getType() in ['List','Item']:
                if callback(nodeInstance.getType(), *args, **kw):
                    return True
        return False
        
    def _compareTypes(self, itemType1, *args, **kw):
        """ Callback used to compare two type names.

            Used by `TypedList.walkOnXMLObject`
        """
        itemType2 = getattr(args[0],'__name__',args[0])
        if itemType1 == '#PCDATA' and type(itemType2) == type(''):
            itemType1 = itemType2
        return itemType1 == itemType2


    def checkItem(self, it):
        parentType = self._xmlList.getItemType()
        typeMismatch = False
        if type(it) == type(''):
            if not self.walkOnXMLObject(parentType,
                                        self._xmlList.getRegistry(),
                                        self._compareTypes, it):
                typeMismatch = True
     
        elif not isinstance(it, XMLObject):
            typeMismatch = True
        elif it.getName() != parentType and \
                 not self.walkOnXMLObject(parentType,
                                          self._xmlList.getRegistry(),
                                          self._compareTypes, it):
            typeMismatch = True
            
        if typeMismatch:
            raise TypeError("""\
%s type required for %s. Got %s instead""" % (`parentType`,
                                              `self._xmlList.getName()`,
                                              `type(it)`))
        if isinstance(it,XMLObject) and it.getName() == self._xmlList.getParentType():
            it = copy.deepcopy(it)
        return it#copy.deepcopy(it)
        
    def append(self, item):
        item = self.checkItem(item)
        UserList.UserList.append(self, item)

    def insert(self, index, item):
        item = self.checkItem(item)
        UserList.UserList.insert(self,index, item)

    def extend(self, other):
        other2 = []
        for it in other:
            other2.append(self.checkItem(it))
        UserList.UserList.extend(self, other2)
